package partie_1;

import java.util.Calendar;
//  INFO-H-301 - POO
//  TP2 - Exercice 2.1, 2.2 et 2.3
//  Date.java


/** 
*   voir nonc: http://cs.ulb.ac.be/public/_media/teaching/infoh301/poo_tp02_exercices.pdf
*   Le traitement des erreurs peut-tre fait soit:
*   - en lanant une Exception (obligation de traiter ou dclarer)
*   - en lanant une Exception Runtime (pas d'obligation de traitement ou dlcaration)
*   - en choisissant une valeur par dfaut en cas d'inconsistence
*/

public class Date {

	private static final int daysInMonths[]={31,28,31,30,31,30,31,31,30,31,30,31};
	private static final String monthInFrench[]={ 
		"Janvier", "Fvrier", "Mars",
		"Avril", "Mai", "Juin", "Juillet", "Aout",
		"Septembre", "Octobre", "Novembre", "Decembre"
		};
	private static final String dayInFrench[]={
	"Lundi", "Mardi", "Mercredi",
	"Jeudi", "Vendredi", "Samedi",
	"Dimanche"
	};
	
	private int day;
	private int month;
	private int year;
	
	/**
	 * Constructs a new Date object representing today. 
	 */	
	public Date()
	{
		Calendar now = Calendar.getInstance(); //Calendar from java.util
		int year = now.get(Calendar.YEAR);
		int month = now.get(Calendar.MONTH)+1; //in Calendar Jan=0, Feb=1...
		int day = now.get(Calendar.DAY_OF_MONTH);
		setYear(year);
		setMonth(month);
		setDay(day);
	}
	
	
	/**
	 * Constructs a new Date object representing the date day-month-year. 
	 * @throws RuntimeException if month is not between 1 and 12.
	 * @throws RuntimeException if day is not a day of the current month
	 */	
	public Date(int day, int month, int year)
	{
		setYear(year);
		setMonth(month);
		setDay(day);
	}
	
	public int getDay()
	{
		return this.day;
	}
	
	public int getMonth()
	{
		return this.month;
	}
	
	public int getYear()
	{
		return this.year;
	}
	
	public void setYear(int year)
	{
		this.year=year;
	}
	
	/**
	 * Sets the month. 
	 * @throws RuntimeException if month is not between 1 and 12.
	 */
	
	public void setMonth(int month)
	{
		if(month<1 || month >12) 
			throw new RuntimeException("Invalid month: "+month);
		this.month = month;
	}
	
	/**
	 * Sets the day. 
	 * Year and Month must be set already BEFORE calling this method.
	 * @throws RuntimeException if day is not a day of the current month
	 */
	public void setDay(int day)
	{
		if(day<1 || day>daysInMonth()) 
			throw new RuntimeException("Invalid day: "+day+" (must be between 1 and "+daysInMonth()+")");
		else 
			this.day=day;
	}
	
	/**
	 *  increments the date of one day
	 **/
	public void increment()
	{
		if(lastDayOfMonth()) {
			day = 1;
			if(month == 12){
				month = 1; 
				year++;
			} else {
				month++;
			}
		} else {
			day++;
		}
	}
	
	/**
	 *  @returns The number of days in the month/year specified.
	 *  @throw RuntimeException if month is not between 1 and 12.
	 **/
	public static int daysInMonth(int month, int year)
	{
		if (month>12 || month <1) 
			throw new RuntimeException("Invalid month: "+month);
		return daysInMonths[month-1] 
		+ (Date.isLeapYear(year) && month==2? 1 : 0 );
	}
	
	/**
	 *  @returns The number of days in the month this date belongs to.
	 **/
	private  int daysInMonth()
	{
		return Date.daysInMonth(this.month, this.year);
	}
	
	/**
	 * @returns true if this is the last day of the month.
	 */
	private boolean lastDayOfMonth()
	{
		return day == daysInMonth();
	}
	
	/**
	 * @returns true if it is a leap year.
	 */
	private boolean isLeapYear()
	{
		return Date.isLeapYear(this.year);
	}
	
	/**
	 * @returns true if year is a leap year.
	 */
	public static boolean isLeapYear(int year)
	{
		return year%400==0 || (year%100 !=0 && year%4==0);
	}
	
	/**
	 * @returns the day number in the current year this date represents.
	 */	
	public int dayOfYear()
	{
		int dayOfYear=this.day;
		for(int i=0; i<month-1;i++)
			dayOfYear += daysInMonths[i];
		if(isLeapYear() && month>2) dayOfYear++;
		return dayOfYear;
	}
	
	/**
	* @returns 0=monday, 1=tuesday, ..., 6=sunday
	* see http://en.wikipedia.org/wiki/Zeller's_congruence
	 */	
	public int dayOfWeek()
	{
		int m = this.month; // local copies because
		int y = this.year;  // month and year can be modified
		if(m==1 || m==2)
		{
			m += 12;
			y--;            
		}
		
		int century = y/100;
		int yearOfCentury = y%100;
		int dayOfWeek = (day + 
						 + (((m+1)*26)/10)
						 + yearOfCentury 
						 + (yearOfCentury/4) 
						 + (century/4) 
						 + 5*century) % 7;
		
		return (dayOfWeek+5)%7;
	}
	
	/**
	 * @return the number of completed year from this date to 'to' 
	 */
	public int yearsTo(Date to)
	{
		int years = to.year - this.year;
		if(to.month < this.month || 
		   (to.month == this.month && to.day < this.day))
			years--;
		return years; 
	}
	
	public String toString()
	{
		return dayInFrench[dayOfWeek()]+" "+day+" "+ 
		       monthInFrench[month-1]+" "+year+ 
			   " le "+dayOfYear()+"-ime jour de l'anne";
	}
	
	public static void main(String args[])
	{
		Date today = new Date(5, 11, 2009);
		System.out.println(today);
		
		Person prof = new Person("Jean", "Dupond", new Date(6, 11, 1965));
		System.out.println(prof);
		try {
		prof.save("prof.txt");
		} catch(java.io.IOException io) {
			io.printStackTrace();
		}
		today.setDay(40); // lance une exception Runtime.
	}
}
